<?php
$title = "moha";
$testMode = false;

cli_set_process_title($title);
file_put_contents("/proc/".getmypid()."/comm",$title);

//Constants
define( "EOL", "\n");
define("Z2M", "zigbee2mqtt");
define("ON", 1);
define("OFF", 0);
define("AUTO", 0);
define("MANUAL", 1);
// log levels
define( "DEBUG", 16);		// => 16
define( "INFO", 1);			// => 1
define( "NOTICE", 2);		// => 2
define( "WARNING", 4);	// => 4
define( "ERROR", 8);			// => 8
define( "ALERT", 32);
define( "ALL", DEBUG | INFO | NOTICE | WARNING | ERROR | ALERT);

declare(ticks = 1);

$listProperties = array("powerSource" => "batterie");
$listPropertiesKeys = array("property");

//global variables
$logLevel = INFO | NOTICE | WARNING | ERROR | ALERT; //ALL;
$notificationLevel = WARNING | ERROR;
$topics = array();				// list of topics
$mids = array();				// list of message IDs
$devices = array();				// array of device objetcs
$indexDevices = array();		// index devices by ieee_address
$hooksList = array();			// list of hooks to be included
$hooks = array();				// array of hooks
$notificationMethods = array();	// array of notification methods objects
$events = array();				// list of event objects
$changed = array();				// list of changed devices
$dbInit = 0;					// flag to indicate that devices db is initializedl
$connected = false;				// connected to MQTT server
$included = false;				// flag indicate scripts are loaded
$nSubscribed = 0;				// Number of topics subsribed
$logFile = "/var/log/moha.log";	// Path of log file
$logFh = null;					// filehandle of log file
$curlErr = 0;					// Number of errors returned by curl
$configDir = "./config";		// default config dir (production value is /etc/moha/)
$hooksInitialized = 0;		// are all hooks initialized ? false/true


if (!init()) exit(1);

// gettext
bindtextdomain("moha", "./locale");
textdomain("moha");

function notify($message)
{
	global $notificationMethods;
	$result = false;
	foreach($notificationMethods as $value)
	{
		//$result =  $result | $value->send($message);
	}
	return $result;
}

function logger($level, $log, $notif = true)
{
	global $logFh, $logLevel, $notificationLevel;
	//echo "=====>>>> $level  => $logLevel => $notificationLevel" . EOL ;
	//echo $log .EOL;
	if ($level & $logLevel)
	{
		fwrite($logFh, "$level : $log" . EOL);
		print ("$level : $log" . EOL);
	}
	$test = $level & $notificationLevel;
	//echo "notif =>" .$notif . EOL;
	if (($test != 0) and ($notif === true))
	{
		if(notify("Moha\n" . $log) === false)
		{
			logger(INFO, _("Notification not sent"), false);
		}
	}
}



logger(DEBUG, _("requiring php modules"), false);
require "class/main.php";
require "class/db.php";
require "class/hook_class.php";
require "utils.php";
require $configDir . "/properties2log.php";
require "mqtt_functions.php";
require "events.php";
require "db_functions.php";

logger(DEBUG, _('assigning variable $client to mosquitto class "client"'), false);
$client = new Mosquitto\Client();

logger(DEBUG, _("Loading stored devices datas"));
loadDB($devices, "moha.db");


// topics definition
listHooks("./topics_callbacks", $hooksList);
if (!empty($hooksList))
{
	foreach ($hooksList as $callback)
	{
		logger(INFO, _("Including ") . $callback, false);
		include $callback;
	}
}

logger(DEBUG, _("requiring config files -> devices_constants.php"), false);
//include predefined file witch define constants for devices
if (is_readable($configDir . "/" . "devices_constants.php"))
{
	include $configDir . "/" . "devices_constants.php";
	//echo "hooklist"; print_r($hooksList); echo EOL;
	logger(INFO, sprintf(_("%s/devices_constants.define found, so it has been included :-)"), $configDir), false);
}else
{
	logger(WARNING, sprintf(_("%s/devices_constants.define not found, so not included :-)"), $configDir), false);
}

// making the list of hooks to include
listHooks("./hooks", $hooksList);

// Program start
logger(DEBUG, _("Program start"), false);
$client = new Mosquitto\Client();

// defining callback functions
$client->onConnect('connectResponse');
$client->onDisconnect('disconnectResponse');
$client->onSubscribe('subscribeResponse');
$client->onUnsubscribe('unsubscribeResponse');
$client->onMessage('messageReceived');
$client->onLog('logger');
$client->onPublish('publishResponse');

// connectong to mqtt server
$client->connect("192.168.1.253", 1883, 5);

logger(INFO, _("Subscribing to bridge"), false);
foreach($topics as $name => $topic)
{
	$topic->mid = $client->subscribe($name . "/#", 2);
	$mids[$topic->mid] = $name;
	$topic->status = false;
}

// starting main loop
logger(INFO, _("Starting loop"), false);
$oneshot = false;
while (true)
{
	$client->loop();			// mqtt server loop()
	if (! $included)			// hooks not already included
	{
		logger(DEBUG, _("Making hooks list"), false);
		getDevicesValues();			// TODO get the values of devices
		if (!empty($hooksList))		// some hooks to include if hooklist is not empty
		{
			foreach ($hooksList as $hook)		// loop to include hooks in hookslist
			{
				logger(INFO, _("Including ") . $hook, false);
				include $hook;
			}
			file_put_contents("moha.devices", print_r($devices, true));   // debugging : save device list
			$included = true;
		}

	}else
	{
		if ($oneshot === false)		//	execute once initialization finished :WARNING hooks can to be not initialized
		{
			logger(DEBUG, _("Oneshot part of loop"), false);
			$oneshot = true;
		}
		checkEvents();
		if($hooksInitialized == 0)			// all hooks are not initialized
		{
			$i = 1;
			foreach($hooks as $hookName => $hook)
			{
				if ($hook->initialized === false)
				{
					logger(WARNING, _("Hook not completely initialized :") . $hookName);
					$i &= $hook->installHooks();
				}
			}
			$hooksInitialized = $i;
		}else
		{
			logger(DEBUG,_("All hooks initialized"));
		}
	}
}

endMoha();

function init()
{
	global $logFile, $logFh, $client;
	date_default_timezone_set('Europe/Paris');
	if (! $logFh = fopen($logFile, "w") )
	{
		echo _("error opening log file");
		return false;
	}
	return true;
}

function listHooks($dir, &$hookList)
{
	$files = scandir($dir);
	foreach ($files as $file)
	{
		if ($file != "." and $file != "..")
		{
			if (is_dir($dir . "/" . $file))
			{
				listHooks($dir . '/' . $file, $hookList);
			}elseif (strpos($file, ".php", -4) !== false)
			{
				if (substr($file, -4) == ".php")
				{
					$hookList[] = $dir . "/" . $file;
				}
			}
		}
	}
}

function endMoha()
{
	global $devices, $topics, $nSubscribed ,$client, $logFh, $connected;
	$x = 0;
	storeDB($devices, "moha.db");
	file_put_contents("moha.devices", print_r($devices, true));
	if ($connected)
	{
		$mid = $client->unsubscribe("#");
		$client->disconnect();
		while ($connected)
		{
			if ( $x++ <= 60)
			{
				fclose($logFh);
				exit (1);
			}
			$client->loop();
		}
	}
	fclose($logFh);
	exit(0);
}



?>
